var colorModule = require("color");
var types = require("utils/types");
var enums = require("ui/enums");
var cssValue = require("css-value");
var Background = (function () {
    function Background(color, image, repeat, position, size) {
        this.color = color;
        this.image = image;
        this.repeat = repeat;
        this.position = position;
        this.size = size;
    }
    Background.prototype.withColor = function (value) {
        return new Background(value, this.image, this.repeat, this.position, this.size);
    };
    Background.prototype.withImage = function (value) {
        return new Background(this.color, value, this.repeat, this.position, this.size);
    };
    Background.prototype.withRepeat = function (value) {
        return new Background(this.color, this.image, value, this.position, this.size);
    };
    Background.prototype.withPosition = function (value) {
        return new Background(this.color, this.image, this.repeat, value, this.size);
    };
    Background.prototype.withSize = function (value) {
        return new Background(this.color, this.image, this.repeat, this.position, value);
    };
    Background.prototype.getDrawParams = function (width, height) {
        if (!this.image) {
            return null;
        }
        var res = {
            repeatX: true,
            repeatY: true,
            posX: 0,
            posY: 0,
        };
        if (this.repeat) {
            switch (this.repeat.toLowerCase()) {
                case enums.BackgroundRepeat.noRepeat:
                    res.repeatX = false;
                    res.repeatY = false;
                    break;
                case enums.BackgroundRepeat.repeatX:
                    res.repeatY = false;
                    break;
                case enums.BackgroundRepeat.repeatY:
                    res.repeatX = false;
                    break;
            }
        }
        var imageWidth = this.image.width;
        var imageHeight = this.image.height;
        if (this.size) {
            var values = cssValue.parse(this.size);
            if (values.length === 2) {
                var vx = values[0];
                var vy = values[1];
                if (vx.unit === "%" && vy.unit === "%") {
                    imageWidth = width * vx.value / 100;
                    imageHeight = height * vy.value / 100;
                    res.sizeX = imageWidth;
                    res.sizeY = imageHeight;
                }
                else if (vx.type === "number" && vy.type === "number" &&
                    ((vx.unit === "px" && vy.unit === "px") || (vx.unit === "" && vy.unit === ""))) {
                    imageWidth = vx.value;
                    imageHeight = vy.value;
                    res.sizeX = imageWidth;
                    res.sizeY = imageHeight;
                }
            }
            else if (values.length === 1 && values[0].type === "ident") {
                var scale = 0;
                if (values[0].string === "cover") {
                    scale = Math.max(width / imageWidth, height / imageHeight);
                }
                else if (values[0].string === "contain") {
                    scale = Math.min(width / imageWidth, height / imageHeight);
                }
                if (scale > 0) {
                    imageWidth *= scale;
                    imageHeight *= scale;
                    res.sizeX = imageWidth;
                    res.sizeY = imageHeight;
                }
            }
        }
        if (this.position) {
            var values = cssValue.parse(this.position);
            var spaceX = width - imageWidth;
            var spaceY = height - imageHeight;
            if (values.length === 2) {
                var vx = values[0];
                var vy = values[1];
                if (vx.unit === "%" && vy.unit === "%") {
                    res.posX = spaceX * vx.value / 100;
                    res.posY = spaceY * vy.value / 100;
                }
                else if (vx.type === "number" && vy.type === "number" &&
                    ((vx.unit === "px" && vy.unit === "px") || (vx.unit === "" && vy.unit === ""))) {
                    res.posX = vx.value;
                    res.posY = vy.value;
                }
                else if (vx.type === "ident" && vy.type === "ident") {
                    if (vx.string.toLowerCase() === "center") {
                        res.posX = spaceX / 2;
                    }
                    else if (vx.string.toLowerCase() === "right") {
                        res.posX = spaceX;
                    }
                    if (vy.string.toLowerCase() === "center") {
                        res.posY = spaceY / 2;
                    }
                    else if (vy.string.toLowerCase() === "bottom") {
                        res.posY = spaceY;
                    }
                }
            }
        }
        return res;
    };
    Background.prototype.isEmpty = function () {
        return types.isNullOrUndefined(this.image) && types.isNullOrUndefined(this.color);
    };
    Background.equals = function (value1, value2) {
        if (!value1 && !value2) {
            return true;
        }
        if (!value1 || !value2) {
            return false;
        }
        return value1.image === value2.image &&
            value1.position === value2.position &&
            value1.repeat === value2.repeat &&
            value1.size === value2.size &&
            colorModule.Color.equals(value1.color, value2.color);
    };
    Background.default = new Background(undefined, undefined, undefined, undefined, undefined);
    return Background;
})();
exports.Background = Background;
